;+
;
; @file_comments
;
; @categories
;
; @keyword IMAGE
;
; @examples
;  
; To run test with Levitus temperature on a regular 1x1 grid :
;   IDL> @tst_initlev
;   IDL> tst_pltt
;
; @history
; Sebastien Masson
;
; @version
; $Id$
;
;-
PRO tst_pltt, IMAGE=image
;
  compile_opt idl2, strictarrsubs
;
@cm_4mesh
@cm_4cal
;
; figure 2: regular grid
;
  CASE jpi*key_stride[0]+jpj*key_stride[1] OF
    180+148:file = 'Levitus98_1m_01_12_Temperature_Pot_ORCA2.nc'
    360+180:file = 'Levitus98_1m_01_12_Temperature_Pot_1x1.nc'
    720+510:file = 'Levitus98_1m_01_12_Temperature_Pot_ORCA05.nc'
  ENDCASE
  file = isafile(file, title = 'Where is '+file+' ?', /lookalldir)
  IF size(file, /type) NE 7 THEN return
;
  dummy = ''
  cnt = 1
;
; -1/1 xt plot
;
  IF key_onearth THEN domdef, 20, 380, -1, 1, 0, gdept[0] $
  ELSE domdef, min(glamt), max(glamf), jpj/2-1, jpj/2+1, 0, gdept[0]
;
  temp = read_ncdf('votemper', 00101, 01231, file = file)
  IF key_forgetold THEN help, temp.arr ELSE help, temp.tab
  help, jpt, time
  pltt, temp
  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png $
  ELSE read, dummy,  prompt = 'press return for the next plot'
  cnt = cnt + 1
;
  pltt, temp, /nocontour
  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png $
  ELSE read, dummy,  prompt = 'press return for the next plot'
  cnt = cnt + 1
;
  pltt, temp, /color_c
  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png $
  ELSE read, dummy,  prompt = 'press return for the next plot'
  cnt = cnt + 1
;
  pltt, temp, /nofill
  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png $
  ELSE read, dummy,  prompt = 'press return for the next plot'
  cnt = cnt + 1
;
; equatorial plot
;
  IF key_onearth THEN BEGIN
    a = abs(gphit[0, *])
    yind = where(a EQ min(a))
    domdef, 20, 380, yind[0], yind[n_elements(yind)-1] $
      , gdept[10 < (jpk-1)], gdept[10 < (jpk-1)], gridtype = 'T', /yindex
  ENDIF ELSE BEGIN
    domdef, min(glamt), max(glamf), jpj/2, jpj/2 $
      , gdept[10 < (jpk-1)], gdept[10 < (jpk-1)], gridtype = 'T', /yindex
  ENDELSE
  temp = read_ncdf('votemper', 00101, 01231, file = file)
  IF key_forgetold THEN help, temp.arr ELSE help, temp.tab
  pltt, temp
  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png $
  ELSE read, dummy,  prompt = 'press return for the next plot'
  cnt = cnt + 1
;
; yt plot along the column that contain the largest latitude
;
  IF key_onearth THEN BEGIN
    ind = where(gphit EQ max(gphit))
    ind = ind[0] MOD jpi
    domdef, ind, ind, -90, 90, gdept[0], gdept[0], gridtype = 'T', /xindex
  ENDIF ELSE BEGIN
    domdef, jpi/2, jpi/2, min(gphit), max(gphif), gdept[0], gdept[0], gridtype = 'T', /xindex
  ENDELSE

  temp = read_ncdf('votemper', 00101, 01231, file = file)
  IF key_forgetold THEN help, temp.arr ELSE help, temp.tab
  pltt, temp

  if keyword_set(image) then saveimage, 'tst_pltt_'+image+'_'+string(cnt, format = '(i2.2)')+'.png', /png
  cnt = cnt + 1

  return
end
